//$Id: text-field-prop.cc,v 1.4 2001/11/03 01:30:45 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "text-field-prop.h"

#include "config.h"
#include <libgnome/libgnome.h>

#include "widget-prop.h"
#include "resizeable-prop.h"
#include "textual-prop.h"


#include "widgets/num-entry.h"
#include "widgets/togglebutton.h"
#include "widgets/propertytable.h"

using namespace Guikachu::GUI::FormEditor;
using namespace Guikachu::Widgets;

TextFieldProperties::TextFieldProperties (TextField *res)
{
    using namespace Gtk;
    using namespace SigC;

    GUI::PropertyTable *proptable = new GUI::PropertyTable ();
    Gtk::Widget *control;
    
    /* Resource ID */
    WidgetProperties::add_general_controls (res, *proptable);

    /* Usable */
    control = new GUI::ToggleButton (res->usable);
    proptable->add (_("_Usable:"), *manage (control),
		    _("Non-usable widgets are not rendered by default"));

    /* Font */
    control = new GUI::NumEntry (0, 2, res->font);
    proptable->add (_("_Font:"), *manage (control));
    
    /* Editable */
    control = new GUI::ToggleButton (res->editable);
    proptable->add (_("_Editable:"), *manage (control));

    /* Multi-line */
    control = new GUI::ToggleButton (res->multi_line);
    proptable->add (_("_Multi-line:"), *manage (control));

    /* Numeric */
    control = new GUI::ToggleButton (res->numeric);
    proptable->add (_("Allow _numeric only:"), *manage (control),
		    _("Numeric text fields only accept numbers as input"));

    /* max_length */
    control = new GUI::NumEntry (0, 32767, res->max_length);
    proptable->add (_("Maximum length:"), *manage (control),
		    _("Maximum allowed text length, in bytes. "
		      "0 means maximum lenght allowed by PalmOS"));

    proptable->add_separator ();
    
    /* Underline */
    control = new GUI::ToggleButton (res->underline);
    proptable->add (_("U_nderlined:"), *manage (control));
    
    /* Auto-shift */
    control = new GUI::ToggleButton (res->auto_shift);
    proptable->add (_("_Auto-shift:"), *manage (control),
		    _("Toggle intelligent shift state managment "
		      "(e.g. turning shift on after a dot)"));

    /* Dynamic size */
    control = new GUI::ToggleButton (res->dynamic_size);
    proptable->add (_("_Dynamic size:"), *manage (control),
		    _("Resize text field based on content's size"));
    
    /* Justify right */
    control = new GUI::ToggleButton (res->justify_right);
    proptable->add (_("_Right-aligned:"), *manage (control),
		    _("Toggle left/right justification. "
		      "Right-justified fields don't accept "
		      "TAB characters"));
    
    /* has_scrollbar */
    control = new GUI::ToggleButton (res->has_scrollbar);
    proptable->add (_("_Scrollbar:"), *manage (control),
		    _("Turn this on to display a vertical scrollbar "
		      "if the text entered is longer than the field's "
		      "visible size"));
    
    /* Separator */
    proptable->add_separator ();

    /* Position */
    WidgetProperties::add_position_controls (res, *proptable);

    /* Size */
    ResizeableProperties::add_controls (res, *proptable);
    
    proptable->show ();
    property_editor = proptable;
}

TextFieldProperties::~TextFieldProperties ()
{
    delete property_editor;
}

Gtk::Widget* TextFieldProperties::get_editor ()
{
    return property_editor;
}
