//$Id: storage-node.cc,v 1.5 2001/11/06 12:54:53 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "storage-node.h"

#include "property.h"

#include <gnome-xml/xmlmemory.h>
#include <glib.h>

using namespace Guikachu;
using std::string;

StorageNode::StorageNode ():
    node_ptr (0)
{
}

StorageNode::StorageNode (xmlNodePtr root):
    node_ptr (root)
{   
}

StorageNode StorageNode::add_node (const string &name,
				   const string &content = "")
{
    xmlNodePtr new_node_ptr = 0;

    if (content != "")
	new_node_ptr = xmlNewChild (node_ptr, 0,
				    (xmlChar*)name.c_str (),
				    (xmlChar*)content.c_str ());
    else
	new_node_ptr = xmlNewChild (node_ptr, 0,
				    (xmlChar*)name.c_str (), 0);

    return StorageNode (new_node_ptr);
}

void StorageNode::set_content (const string &content)
{
    xmlNodeSetContent (node_ptr, (xmlChar*) content.c_str ());
}

string StorageNode::get_content () const
{
    string ret;
    char *text = (char*)xmlNodeGetContent (node_ptr);
    
    if (text)
    {
	ret = text;
	xmlFree (text);
    } else {
	ret = "";
    }

    return ret;
}


void StorageNode::set_prop (const string &prop_name,
			    const string &value)
{
    xmlSetProp (node_ptr,
		(xmlChar*) prop_name.c_str (),
		(xmlChar*) value.c_str ());
}

void StorageNode::set_prop (const string &prop_name,
			    unsigned int  value)
{
    char *tmp = g_strdup_printf ("%d", value);
    set_prop (prop_name, tmp);
    g_free (tmp);
}

void StorageNode::set_prop (const string &prop_name,
			    int           value)
{
    char *tmp = g_strdup_printf ("%d", value);
    set_prop (prop_name, tmp);
    g_free (tmp);
}

void StorageNode::set_prop (const string &prop_name,
			    char          value)
{
    if (!value)
	return;
    
    static char *buffer = g_new0 (char, 2);

    buffer[0] = value;
    set_prop (prop_name, buffer);
}

template <>
string StorageNode::get_prop <string> (const string &prop_name) const
{
    string ret;
    
    xmlChar *xml_tmp = xmlGetProp (node_ptr, (xmlChar*) prop_name.c_str ());

    if (xml_tmp)
    {
	ret = (char*) xml_tmp;
	xmlFree (xml_tmp);
    } else {
	ret = "";
    }

    return ret;
}

template <>
int StorageNode::get_prop <int> (const string &prop_name) const
{
    int ret;

    xmlChar *xml_tmp = xmlGetProp (node_ptr, (xmlChar*) prop_name.c_str ());
    if (xml_tmp)
    {
	ret = atoi ((char*) xml_tmp);
	xmlFree (xml_tmp);
    } else {
	ret = 0;
    }

    return ret;
}

template <>
char StorageNode::get_prop <char> (const string &prop_name) const
{
    char ret = 0;

    xmlChar *xml_tmp = xmlGetProp (node_ptr, (xmlChar*) prop_name.c_str ());
    if (xml_tmp)
    {
	if (strlen ((char*) xml_tmp))
	    ret = (char) xml_tmp[0];
	
	xmlFree (xml_tmp);
    }

    return ret;
}
